@@ -1,8 +1,7 @@
-from .Constituent import Constituent
+from .Constituent import Constituent,_getElems
 from .Terminal import Terminal, N, A, Pro, D, Adv, V, P, C, DT, NO, Q
 from .Phrase import Phrase, prepositionsList
 from .Lexicon import getLexicon, getRules, currentLanguage
-import sys
 
 class Dependent(Constituent):
     def __init__(self, params, deprel, lang=None):
@@ -20,6 +19,8 @@ class Dependent(Constituent):
         params=list(params)
         if isinstance(params[0],Terminal):
             self.terminal=params.pop(0)
+        elif isinstance(params[0],str):
+            self.terminal=Q(params.pop(0))
         else:
             self.warn("Dependent needs Terminal",type(params[0]).__name__)
             params.pop(0)
@@ -28,7 +29,7 @@ class Dependent(Constituent):
             self.peng=self.terminal.peng
         if self.terminal.isA("V"):
             self.taux=self.terminal.taux
-        params = [e for e in params if e != None]  # ignore None dependents
+        params = _getElems(params)  # flatten dependents and ignore None dependents
         # list of dependents to create the source of the parameters at the time of the call
         # self can be different from the dependents lists because of structure modifications
         self.dependentsSource = []
@@ -40,7 +41,7 @@ class Dependent(Constituent):
                 self.addDependent(d)
                 self.dependentsSource.append(d)
             else:
-                self.warn("bad Dependent", NO(i + 1).dOpt({"ord":True}), type(d).__name__+str(d))
+                self.warn("bad Dependent", NO(i + 2).dOpt({"ord":True}), type(d).__name__+":"+str(d))
         if len(params) > 0:
             # terminate the list with add which does other checks on the final list
             self.add(params[-1], None, True)
@@ -49,7 +50,7 @@ class Dependent(Constituent):
         if isinstance(dependent,Dependent):
             dependent.parentConst=self
             # add it to the list of dependents
-            if position==None:
+            if position is None:
                 self.dependents.append(dependent)
             elif isinstance(position,int) and position <= len(self.dependents) and position>=0:
                 self.dependents.insert(position,dependent)
@@ -101,6 +102,15 @@ class Dependent(Constituent):
             args+=self.dependents
         return f'{self.constType}({",".join([e.me() for e in args])})'
 
+    def setPengRecursive(self,oldDep,oldPengNO,newPeng):
+        oldDep.peng = newPeng
+        # check if terminal and children of oldDep have the same oldPengNO and change it to the newPeng
+        if hasattr(oldDep.terminal,"peng") and oldDep.terminal.peng["pengNO"] == oldPengNO:
+            oldDep.terminal.peng=newPeng
+        for d in oldDep.dependents:
+            self.setPengRecursive(d,oldPengNO,newPeng)
+
+
     def linkProperties(self):
         if len(self.dependents)==0:return self
         # loop over children to set the peng and taux to their head or subject
@@ -109,8 +119,11 @@ class Dependent(Constituent):
         if self.isA("coord"):
             # must create self.peng already because it might be used in the current dependents (for adjectives, attributes...)
             # the information will be computed at realization time (see Dependent.coordReal)
-            self.peng={}
-        for d in self.dependents:
+            Constituent.pengNO += 1
+            self.peng={"pengNO":Constituent.pengNO}
+            headTerm.peng = self.peng
+        for i in range(0,len(self.dependents)): # should use "for d in dependents:"
+            d = self.dependents[i]              # but this "kills" the PyCharm debugger
             depTerm=d.terminal
             deprel=d.constType
             if deprel=="subj":
@@ -119,26 +132,26 @@ class Dependent(Constituent):
             elif deprel=="det":
                 if depTerm.isA("D"):
                     if hasattr(self,"peng"):
-                        if hasattr(depTerm,"peng") and hasattr(depTerm.peng,"pe"): # save person (for possessives)
+                        if hasattr(depTerm,"peng") and "pe" in depTerm.peng: # save person (for possessives)
                             pe=depTerm.peng["pe"]
                             depTerm.peng=self.peng
                             depTerm.peng["pe"]=pe
                         else: #some strange determiner construct do not have peng
                             depTerm.peng=self.peng
                 elif depTerm.isA("NO"):
-                    headTerm.peng["n"]=depTerm.grammaticalNumber()
-                    # gender agreement between a French numbeer and subject
-                    depTerm.peng["g"]=headTerm.peng["g"]
-                elif depTerm.isA("P") and depTerm.lemma=="de": # HACK: deal with specific case : det(P("de"),mod(D(...)))
-                    if len(d.dependents)==1 and d.dependents[0].isA("mod") and d.dependents[0].terminal.isA("D"):
-                        d.dependents[0].terminal.peng = self.peng
+                    depTerm.peng=headTerm.peng
             elif deprel=="mod" or deprel=="comp":
-                if depTerm.isA("A"):
+                if depTerm.isA("A") or (depTerm.isA("V") and depTerm.getProp("t")=="pp"):
                     depTerm.peng=self.peng
-                    # check for an attribute of a copula with an adjective
+                    # check for an attribute of a copula with an adjective or past participle
                     if self.isFr() and headTerm.lemma in ["être", "paraître", "sembler", "devenir", "rester"]:
-                        iSubj=self.findIndex(lambda d0:d0.isA("subj") and d0.terminal.isA("N"))
-                        if iSubj>=0: depTerm.peng=self.dependents[iSubj].peng
+                        iSubj=self.findIndex(lambda d0:d0.isA("subj") and d0.terminal.isOneOf(["N","Pro"]))
+                        if iSubj>=0:
+                            depTerm.peng=self.dependents[iSubj].peng
+                        iAttr=self.findIndex(lambda d0:d0.isA("comp") and
+                                                       (d0.terminal.isA("A") or (d0.terminal.isA("V") and d0.terminal.getProp("t")=="pp")))
+                        if iAttr>=0:
+                            self.dependents[iAttr].peng=depTerm.peng
                 elif depTerm.isA("V"):
                     # set agreement between the subject of a subordinate or the object of a relative subordinate
                     iRel=d.findIndex(lambda d0:d0.isOneOf(["subj","comp","mod"]) and
@@ -153,39 +166,51 @@ class Dependent(Constituent):
                     # check for past participle in French that should agree with the head
                     if self.isFr() and depTerm.getProp("t")=="pp":
                         depTerm.peng=self.peng
+                elif depTerm.isA("Pro") and depTerm.lemma in ["qui","que","who","that"]:
+                    # a relative linked to depTerm in which the new peng should be propagated
+                    depTerm.peng = self.peng
+                    for d0 in d.dependents:
+                        self.setPengRecursive(d0,d0.peng["pengNO"],self.peng)
             elif deprel=="root":
-                self.error("An internal root was found")
+                # self.error("An internal root was found")
+                pass
             elif deprel=="coord":
                 if len(d.dependents)>0:
                     firstDep=d.dependents[0]
                     if firstDep.isA("subj"):
                         headTerm.peng=d.peng
-                    elif firstDep.isOneOf(["mod","comp"]) and firstDep.terminal.isA("V"):
-                        # consider this as coordination of verb sharing a subject (the current root)
-                        for d0 in d.dependents:
-                            d0.peng=self.peng
-                            d0.terminal.peng=self.peng
+                    elif firstDep.isA("det"):
+                        d.peng=headTerm.peng
+                    elif firstDep.isOneOf(["mod","comp"]) and firstDep.terminal.isOneOf(["V","A"]):
+                        # consider this as a coordination of verb sharing a subject (the current root)
+                        #            or as a coordination of adjectives
+                        if hasattr(headTerm,"peng"):
+                            for d0 in d.dependents:
+                                d0.peng=headTerm.peng
+                                d0.terminal.peng=headTerm.peng
             else:
-                self.error("Strange dependent:"+d.constType)
+                self.error("Strange dependent:"+deprel)
+        return
 
-    def setLemma(self,lemma,terminalTyp):
+    def setLemma(self,_lemma,_terminalTyp):
         self.error("***: should never happen: setLemma: called on a Dependent")
         return self
 
     def findGenderNumberPerson(self,andCombination):
-        g="f"
+        g=None
         n="s"
         pe=3
         nb=0
         for i in range(0,len(self.dependents)):
             e=self.dependents[i].terminal
-            if e.isOneOf(["NP","N","Pro","Q"]):
+            if e.isOneOf(["NP","N","Pro","Q","NO"]):
                 nb += 1
                 propG=e.getProp("g")
-                if propG=="m" or propG=="x" or e.isA("Q"):g="m" # masculine if gender is unspecified
+                if g is None and propG is not None: g=propG
+                if propG=="m": g="m" # masculine if one masculine is encountered
                 if e.getProp("n")=="p":n="p"
                 propPe=e.getProp("pe")
-                if propPe != None and propPe<pe:pe=propPe
+                if propPe is not None and propPe<pe:pe=propPe
         if nb==0: g="m"
         elif nb>1 and n=="s" and andCombination:n="p"
         return {"g":g,"n":n,"pe":pe}
@@ -217,7 +242,7 @@ class Dependent(Constituent):
                     return self.warn("no appropriate pronoun")
         else:
             pro = self.getTonicPro("acc")
-            if self.parentConst != None and self.parentConst.terminal.isA("V"):  # consider that it is direct complement
+            if self.parentConst is not None and self.parentConst.terminal.isA("V"):  # consider that it is direct complement
                 self.parentConst.terminal.cod = self  # indicate that self is a COD
         # replace the original with the pronoun
         pro.parentConst = self
@@ -232,7 +257,7 @@ class Dependent(Constituent):
     #  Does not currently deal with "Give the book to her." that {c|sh}ould be "Give her the book."
     def pronominalize_en(self):
         self.props["pe"] = 3  # ensure that pronominalization of a noun is 3rd person
-        if self.isA("subj"):  # is it a subject
+        if self.parentConst is None or self.isA("subj"):  # is it a subject
             pro = self.getTonicPro("nom")
         else:
             pro = self.getTonicPro("acc")  # is direct complement
@@ -264,10 +289,7 @@ class Dependent(Constituent):
             if subjIdx>=0:
                 subj=self.dependents[subjIdx]
                 if subj.terminal.isA("Pro"):
-                    # as self pronoun will be preceded by "par" or "by", the "bare" tonic form is needed
-                    # to which we assign the original person, number, gender
-                    subj.terminal=subj.terminal.getTonicPro().g(subj.getProp("g"))\
-                                            .n(subj.getProp("n")).pe(subj.getProp("pe"))
+                    subj.terminal = subj.terminal.getTonicPro()
             else:
                 subj=None
             # find direct object (first N or Pro of a comp) from dependents
@@ -287,31 +309,41 @@ class Dependent(Constituent):
                 if subj is not None:   # the original subject is now the indirect object
                     subj.changeDeprel("mod")
                     self.removeDependent(subjIdx)
-                    self.addDependent(comp(P("par" if self.isFr() else "by",self.lang),subj))
+                    if subj.terminal.isA("V"):  # the subject is a V
+                        prep = "de" if self.isFr() else "to"
+                    else:
+                        prep = "par" if self.isFr() else "by"
+                    self.addDependent(comp(P(prep,self.lang),subj))
             elif subj is not None: # no object, but with a subject
-                #create a dummy subject with a "il"/"it"
+                # create a dummy subject with a "il"/"it"
                 obj=Pro("lui" if self.isFr() else "it",self.lang).c("nom") #HACK: obj is the new subject
                 # add new subject at the front of the sentence
+                subj.changeDeprel("mod")
+                self.removeDependent(subjIdx)
                 self.addPre(obj)
                 self.peng=obj.peng
-                 # add original subject after the verb to serve as an object
-                self.addDependent(comp(P("par" if self.isFr() else "by",self.lang),subj))
+                # add original subject after the verb to serve as an object
+                if subj.terminal.isA("V"):  # the subject is a V
+                    prep = "de" if self.isFr() else "to"
+                else:
+                    prep = "par" if self.isFr() else "by"
+                self.addDependent(comp(P(prep,self.lang),subj))
             if self.isFr():
                 # do self only for French because in English self is done by processTyp_en
                 # change verbe into an "être" auxiliary and make it agree with the newSubj
                 # force person to be 3rd (number and tense will come from the new subject)
                 verbe=self.terminal.lemma
-                self.terminal.setLemma("être")
+                self.terminal.setLemma("avoir" if verbe == "être" else "être")
                 self.terminal.pe(3)
                 if self.getProp("t")=="ip":
-                    self.t("s") # set subjonctive present tense for an imperativ
+                    self.t("s") # set subjonctive present tense for an imperative
                 pp = V(verbe,"fr").t("pp")
                 if obj is not None: # self can be undefined when a subject is Q or missing
                     self.terminal.peng=obj.peng
                     pp.peng=obj.peng
                 # insert the pp before the comp, so that it appears immediately after the verb
                 #  calling addPost(pp) would not put it at the right place
-                compIdx=self.findIndex(lambda d:d.isA("comp"))
+                compIdx=self.findIndex(lambda d:d.isOneOf(["comp","mod"]))
                 if compIdx==-1: compIdx=0
                 self.addDependent(Dependent([pp],"*post*"),compIdx)
         else:
@@ -358,7 +390,7 @@ class Dependent(Constituent):
             rules = getRules()
             origLemma = deprel.terminal.lemma
             for key in rules["verb_option"]["modalityVerb"]:
-                if (key.startswith(mod)):
+                if key.startswith(mod):
                     deprel.terminal.setLemma(rules["verb_option"]["modalityVerb"][key])
                     break
             deprel.terminal.isMod = True
@@ -377,7 +409,7 @@ class Dependent(Constituent):
     def processTyp_en(self, types):
         # replace current verb with the list new words
         #  TODO: take into account the fact that there might be already a verb with modals...
-        if ("contr" in types and types["contr"] is not False):
+        if "contr" in types and types["contr"] is not False:
             # necessary because we want the negation to be contracted within the VP before the S or SP
             self.contraction = True
         words = Phrase.affixHopping(self.terminal, self.getProp("t"), getRules()["compound"], types)
@@ -391,10 +423,15 @@ class Dependent(Constituent):
 
     def moveAuxToFront(self):
         auxIdx = self.findIndex(lambda d: d.isA("*pre*"))  # added by affixHopping
-        if auxIdx >= 0:
+        if auxIdx >= 0 and self.getProp("t") not in ["pp","pr"]: # do not move when tense is participle
             aux = self.dependents[auxIdx].terminal
             self.removeDependent(auxIdx)
             self.addPre(aux, 0)  # put auxiliary before
+        elif self.terminal.lemma in ["be","have"]:
+            # no auxiliary, but check for have or be "alone" for which the subject should appear after the aux
+            subjIdx=self.findIndex(lambda d:d.isA("subj"))
+            if subjIdx>=0:
+                self.dependents[subjIdx].pos("post")
 
     def invertSubject(self):
         # in French : use inversion rule which is quite "delicate"
@@ -404,7 +441,6 @@ class Dependent(Constituent):
         subjIdx = self.findIndex(lambda d: d.isA("subj"))
         if subjIdx >= 0:
             subj = self.dependents[subjIdx].terminal
-            pro = None
             if subj.isA("Pro"):
                 pro = self.removeDependent(subjIdx).terminal  # remove subject
             elif subj.isA("C"):
@@ -416,7 +452,8 @@ class Dependent(Constituent):
                 self.addPost(pro)
                 self.terminal.lier()
 
-    def processTypInt(self,int_):
+    def processTypInt(self,types):
+        int_=types["int"]
         sentenceTypeInt = getRules()["sentence_type"]["int"]
         intPrefix = sentenceTypeInt["prefix"]
         prefix = None  # to be filled later
@@ -436,23 +473,29 @@ class Dependent(Constituent):
                 self.removeDependent(subjIdx)
             prefix = intPrefix[int_]
         elif int_ in ["wod", "wad"]:  # remove direct object (first comp starting with N)
-            for i in range(0, len(self.dependents)):
+            cmp=None
+            i=0
+            while i<len(self.dependents) and cmp is None:
                 d = self.dependents[i]
                 if d.isA("comp") and d.terminal.isA("N"):
-                    self.removeDependent(i)
+                    cmp=self.removeDependent(i)
+                i+=1
             if self.isFr():  # check for passive subject starting with par
                 parIdx = self.findIndex(
                     lambda d: d.isA("comp") and d.terminal.isA("P") and d.terminal.lemma == "par")
                 if parIdx >= 0:
                     pp = self.dependents[parIdx].terminal
                     self.removeDependent(parIdx)  # remove the passive subject
-            prefix = intPrefix[int_]
+            if self.isEn() and int_ == "wod" and cmp is not None and cmp.getProp("g") in ["m","f"]:
+                # human direct object
+                prefix = "whom"
+            else:
+                prefix = intPrefix[int_]
             if self.isEn():
                 self.moveAuxToFront()
             else:
                 self.invertSubject()
         elif int_ in ["woi", "wai", "whe", "whn"]:  # remove indirect object first comp or mod with a P as terminal
-            remove = False
             prefix = intPrefix[int_]  # get default prefix
             for i in range(0, len(self.dependents)):
                 d = self.dependents[i]
@@ -477,11 +520,72 @@ class Dependent(Constituent):
                 self.moveAuxToFront()
             else:
                 self.invertSubject()
+        elif int_=="tag":
+            # according to Antidote: Syntax Guide - Question tag
+            # Question tags are short questions added after affirmations to ask for verification
+            if self.isFr(): # in French really simple, add "n'est-ce pas"
+                self.a(", n'est-ce pas")
+            else: # in English, sources: https://www.anglaisfacile.com/exercices/exercice-anglais-2/exercice-anglais-95625.php
+                # must find  the pronoun and conjugate the auxiliary
+                # look for the first verb or auxiliary (added by affixHopping)
+                vIdx = self.findIndex(lambda d:d.terminal.isA("V") and d.depPosition()=="pre")
+                currV = self.terminal if vIdx<0 else self.dependents[vIdx].terminal
+                if "mod" in types and types["mod"] != False:
+                    aux=getRules("en")["compound"][types["mod"]]["aux"]
+                else:
+                    if currV.lemma in ["have","be","can","will","shall","may","must"]:
+                        aux=currV.lemma
+                    else:
+                        aux="do"
+                neg = "neg" in types and types["neg"]==True
+                pe=currV.getProp("pe")
+                t = currV.getProp("t")
+                n = currV.getProp("n")
+                g = currV.getProp("g")
+                pro = Pro("I").pe(pe).n(n).g(g)  # default pronoun
+                subjIdx = self.findIndex(lambda d:d.isA("subj"))
+                # check for negative adverb
+                advIdx = self.findIndex(lambda d: d.isA("mod") and d.terminal.isA("Adv"))
+                if advIdx>=0 and self.dependents[advIdx].terminal.lemma in ["hardly","scarcely","never","seldom"]:
+                    neg=True
+                if subjIdx >=0 :
+                    subj=self.dependents[subjIdx].terminal
+                    if subj.isA("Pro"):
+                        if subj.getProp("pe")==1 and aux=="be" and t=="p" and not neg:
+                            pe=2 # I am => aren't I
+                        elif subj.lemma in ["this","that","nothing"]:
+                            pro=Pro("I").g("n") # it
+                        elif subj.lemma in ["somebody","anybody","nobody","everybody",
+                                            "someone","anyone","everyone"] :
+                            pro=Pro("I").n("p") # they
+                            if subj.lemma=="nobody":neg=True
+                        else:
+                            pro=subj.clone()
+                        pro=comp(pro)
+                    else:
+                        # must wrap into comp("",...) to force pronominalization of children
+                        pro=comp("",self.dependents[subjIdx].clone().pro())
+                iDeps=len(self.dependents)-1
+                while iDeps>=0 and self.dependents[iDeps].depPosition()!="post":
+                    iDeps-=1
+                if iDeps<0: # add comma to the verb
+                    currV.a(",")
+                else: #add comma to the last current dependent
+                    self.dependents[iDeps].a(",")
+                # nice use-case of jsRealB using itself for realization
+                if aux=="have" and not neg:
+                    # special case because it should be realized as "have not" instead of "does not have"
+                    self.addDependent(comp(V("have").t(t).pe(pe).n(n),
+                                           mod(Adv("not")),pro).typ({"contr":True}))
+                else:
+                    self.addDependent(comp(V(aux).t(t).pe(pe).n(n),
+                                           pro).typ({"neg":not neg,"contr":True}))
+            prefix=intPrefix[int_]
         else:
             self.warn("not implemented", "int:" + int_)
         if self.isFr() or (int_ != "yon"):   # add the interrogative prefix
             self.addPre(Q(prefix), 0)
-        if pp != None:  # add "par" in front of some French passive interrogative
+        if pp is not None:  # add "par" in front of some French passive interrogative
             self.addPre(pp, 0)
         if int_ == "wad":  # replace "que" by "quoi" for French passive wad
             self.dependents[1].terminal.lemma = "quoi"
@@ -498,7 +602,7 @@ class Dependent(Constituent):
         else:
             self.processTyp_en(types)
         if "int" in types and types["int"] is not False:
-            self.processTypInt(types["int"])
+            self.processTypInt(types)
         if "exc" in types and types["exc"] is True:
             self.a(getRules()["sentence_type"]["exc"]["punctuation"], True)
         return self
@@ -517,15 +621,18 @@ class Dependent(Constituent):
             self.setProp("g",dep.getProp("g"))
             self.setProp("n",dep.getProp("n"))
             pe=dep.getProp("pe")
-            self.setProp("pe",pe if pe!=None else 3)
+            self.setProp("pe",pe if pe is not None else 3)
             return self.doFormat(res) # process format for the CP
         # check that all dependents use the same deprel
         deprel=self.dependents[0].constType
+        noConnect = self.terminal.lemma == ""
         for j in range(0,last): #insert comma after each element
             dj=self.dependents[j]
             if not dj.isA(deprel):
                 self.warn("inconsistent dependents within a coord",deprel,dj.constType)
-            if j<last-1: dj.props["a"]=[","]
+            if noConnect or j<last-1:
+                if "a" not in dj.props or "," in dj.props["a"]:
+                    dj.props["a"]=[","]
             res.extend(dj.real())
         # insert realisation of the terminal before last...
         res.extend(self.terminal.real())
@@ -538,25 +645,25 @@ class Dependent(Constituent):
         if selfCoord.isA("C"):
             andC="et" if self.isFr() else "and"
             gn=self.findGenderNumberPerson(selfCoord.lemma==andC)
-            self.setProp("g",gn["g"])
+            if gn["g"] is not None:
+                self.setProp("g",gn["g"])
             self.setProp("n",gn["n"])
             self.setProp("pe",gn["pe"])
             # for an inserted pronoun, we must override its existing properties...
             if  hasattr(selfCoord,"pronoun"):
                 selfCoord.pronoun.peng=gn
-                selfCoord.pronoun.props["g"]=gn["g"]
+                selfCoord.pronoun.props["g"]=gn["g"] if gn["g"] is not None else "m"
                 selfCoord.pronoun.props["n"]=gn["n"]
                 selfCoord.pronoun.props["pe"]=gn["pe"]
         elif not selfCoord.isA("Q"):
             # in some cases the coordination can be a quoted string (possibly empty)
             self.warn("bad parameter","C",selfCoord.constType)
-        return res
+        return self.doFormat(res)
 
     def depPosition(self):
-        myPos = self.props["pos"] if "pos" in self.props else None
-        if myPos=="pre": return "pre"
+        if "pos" in self.props: return self.props["pos"]
         pos="post" # default is post
-        if self.isOneOf(["subj","det","*pre*"]) and myPos!="post":
+        if self.isOneOf(["subj","det","*pre*"]):
             # subject and det are always pre except when specified
             pos="pre"
         elif self.isA("mod") and self.terminal.isA("A") and self.parentConst.terminal.isA("N"):
@@ -565,29 +672,49 @@ class Dependent(Constituent):
                 pos=self.terminal.props["pos"] if "pos" in self.terminal.props else "post"
             else:
                 pos="pre" # all English adjectives are pre
-        elif self.isA("coord"):
-            if myPos is not None:
-                pos=myPos
-            elif len(self.dependents)>0:
-                pos=self.dependents[0].depPosition() #take the position of the first element of the coordination
+        elif self.isA("coord") and len(self.dependents)>0:
+            pos=self.dependents[0].depPosition() #take the position of the first element of the coordination
         return pos
 
     def real(self):
-        self.pronominalizeChildren()
-        if "typ" in self.props:
-            self.processTyp(self.props["typ"])
-        # realize coordinations before anything elso to compute their final number and person
-        for d in self.dependents:
-            if d.isA("coord"):d.tokens=d.coordReal()
-        before=[]
-        after=[]
-        # realize and order them by gathering the dependents that should appear before and after the terminal
-        for d in self.dependents:
-            if d.depPosition()=="pre":
-                before += d.tokens if d.isA("coord") else d.real()
+        if self.isA("coord") and self.parentConst is None:
+            # special case of coord at the root
+            res=self.coordReal()
+        else:
+            self.pronominalizeChildren()
+            if "typ" in self.props:
+                self.processTyp(self.props["typ"])
+
+            # move all "pre" dependents at the front
+            # HACK: we must move these in place because realization might remove some of them
+            nextPre = 0
+            for i in range(0,len(self.dependents)):
+                d = self.dependents[i]
+                if d.depPosition()=="pre":
+                    if nextPre != i:
+                        save = self.dependents[i]
+                        del self.dependents[i]
+                        self.dependents[nextPre:nextPre] = [save]
+                    nextPre += 1
+            # realize dependents
+            if len(self.dependents) == 0:
+                res = self.terminal.real()
+            elif nextPre == 0:  # no pre
+                res = self.terminal.real()
+                for d in self.dependents:
+                    res.extend(d.coordReal() if d.isA("coord") else d.real())
             else:
-                after += d.tokens if d.isA("coord") else d.real()
-        res=[*before,*self.terminal.real(),*after]
+                res = []
+                i=0
+                # cannot use "for i in range()" because dependents list might change during realization
+                while i<len(self.dependents):
+                    d = self.dependents[i]
+                    res.extend(d.coordReal() if d.isA("coord") else d.real())
+                    if i == nextPre-1:
+                        res.extend(self.terminal.real())
+                    i += 1
+            if self.terminal.isA("V"):
+                self.checkAdverbPos(res)
         return self.doFormat(res)
 
     # recreate a jsRealB expression
@@ -614,7 +741,7 @@ class Dependent(Constituent):
         return res
 
     @classmethod
-    def fromJSON(self, constType, json, lang):
+    def fromJSON(cls, constType, json, lang):
         from .utils import fromJSON
         if "terminal" not in json:
             print("Dependent.fromJSON: no terminal found in Dependent:"+str(json))
@@ -638,4 +765,3 @@ def compObj(*params): return Dependent(params,"comp")
 def compObl(*params): return Dependent(params,"comp")
 
 def coord(*params): return Dependent(params,"coord")
-
