@@ -1,4 +1,4 @@
-from .Constituent import Constituent
+from .Constituent import Constituent,_getElems
 from .Terminal import Terminal, N, A, Pro, D, Adv, V, P, C, DT, NO, Q
 from .Lexicon import getLexicon, getRules, currentLanguage
 import sys
@@ -32,11 +32,13 @@ prepositionsList = {
 negMod = {"can": "cannot", "may": "may not", "shall": "shall not", "will": "will not", "must": "must not",
           "could": "could not", "might": "might not", "should": "should not", "would": "would not"}
 
-
 class Phrase(Constituent):
     def __init__(self, constType, elements, lang=None):
-        elements = [e for e in elements if e != None]  # ignore None elements
         super().__init__(constType)
+        if (len(elements) == 0):  # check for an empty list that can be added to
+            elements = []
+        else:  # # transform the tuple received as star args into a list that can be modified
+            elements = _getElems(elements)
         if lang is None:
             self.lang = currentLanguage()
         elif lang in ["en", "fr"]:
@@ -57,7 +59,7 @@ class Phrase(Constituent):
                 self.elements.append(e)
                 self.elementsSource.append(e)
             else:
-                self.warn("bad Constituent", i + 1, type(e).__name__, str(e))
+                self.warn("bad Constituent", i + 1, type(e).__name__)
         if len(elements) > 0:
             # terminate the list with add which does other checks on the final list
             self.add(elements[-1], None, True)
@@ -67,14 +69,14 @@ class Phrase(Constituent):
         if isinstance(elem, Constituent):
             elem.parentConst = self
             # add it to the list of elements
-            if position == None:
+            if position is None:
                 self.elements.append(elem)
             elif isinstance(position, int) and 0 <= position <= len(self.elements):
                 self.elements.insert(position, elem)
             else:
                 self.warn("bad position", position, len(self.elements))
         else:
-            self.warn("bad Constituent", NO(position + 1).dOpt({"ord": True}), type(elem).__name__)
+            self.warn("bad Constituent", NO(position + 1).dOpt({"ord": True}), type(elem).__name__+":"+str(elem))
         return self
 
     # remove a child from this Phrase and return it
@@ -91,6 +93,14 @@ class Phrase(Constituent):
             if start > end: (end, start) = (start, end)
             return all(el.isOneOf(["A", "N"]) for el in elems[start:end + 1])
 
+        if isinstance(constituent,list):
+            if len(constituent)==0: return self
+            if len(constituent)==1:
+                constituent=constituent[0]
+            else:
+                for c in constituent:
+                    self.add(c,position,prog)
+                return self
         if prog is None and constituent is None: return self
         if isinstance(constituent, str):
             constituent = Q(constituent)
@@ -98,7 +108,7 @@ class Phrase(Constituent):
             return self.warn("bad Constituent", "last" if self.isEn() else "dernier",
                              type(constituent).__name__ + ":" + str(constituent))
         if prog is None:
-            self.optSource += f'.add({constituent.toSource(0)}{"" if position is None else ("," + str(position))})'
+            self.optSource += f'.add({constituent.toSource()}{"" if position is None else ("," + str(position))})'
         else:  # call from the constructor
             self.elementsSource.append(constituent)
         constituent.parentConst = self
@@ -109,10 +119,11 @@ class Phrase(Constituent):
             e = self.elements[i]
             if e.isA("A"):  # check for adjective position
                 idx = self.getIndex("N")
-                if e.isFr():
-                    pos = e.props["pos"] if "pos" in e.props else "post"
+                # unless specified the position of an English adjective is pre, but is post for a French one
+                if "pos" in e.props:
+                    pos = e.props["pos"]  
                 else:
-                    pos = "pre"
+                    pos = "pre" if e.isEn() else "post"
                 if idx >= 0:
                     if (pos == "pre" and i > idx) or (pos == "post" and i < idx):
                         if allAorN(self.elements, i, idx):
@@ -130,6 +141,37 @@ class Phrase(Constituent):
             headIndex = 0
         return headIndex
 
+    def linkAttributes(self,vpv,vpcp,subject):
+        if self.isFr() and vpv.lemma in ["être", "paraître", "sembler", "devenir", "rester"]:
+            # check for a coordination of attributes or past participles
+            # vpcp = self.getFromPath([["VP"], ["CP"]])
+            if vpcp is not None:
+                for e in vpcp.elements:
+                    if e.isA("A"):
+                        e.peng = subject.peng
+                    elif e.isA("V") and e.getProp("t") == "pp":
+                        e.peng = subject.peng
+                    elif e.isA("AP"):
+                        e.linkPengWithSubject("AP", "A", subject)
+                    elif e.isA("VP"):
+                        v = e.getConst("V")
+                        if v is not None and v.getProp("t") == "pp":
+                            v.peng = subject.peng
+            else:
+                # check for a single French attribute of a copula verb
+                # with an adjective
+                attribute = vpv.parentConst.linkPengWithSubject("AP", "A", subject)
+                if attribute is None:
+                    elems = vpv.parentConst.elements
+                    try:
+                        vpvIdx = elems.index(vpv)
+                        for i in range(vpvIdx + 1, len(elems)):
+                            e = elems[i]
+                            if e.isA("V") and e.getProp("t") == "pp":
+                                e.peng = subject.peng
+                    except ValueError:
+                        self.error("linkProperties    : verb not found")
+
     def linkProperties(self):
         #  loop over children to set the peng and taux to their head or subject
         #  so that once a value is changed this change will be propagated correctly...
@@ -138,30 +180,37 @@ class Phrase(Constituent):
             # the head is the first internal N, number with a possible NO
             # find first NP or N
             headIndex = self.getHeadIndex("NP")
-            self.peng = self.elements[headIndex].peng
-            for i in range(0, len(self.elements)):
-                if i != headIndex:
-                    e = self.elements[i]
-                    if hasattr(self, "peng"):  # do not try to modify if current peng does not exist e.g. Q
-                        if e.isA("NO") and i < headIndex:  # NO must appear before the N for agreement
-                            self.peng["n"] = e.grammaticalNumber()
-                            # gender agreement between a French number and subject
-                            e.peng["g"] = self.peng["g"]
-                        elif e.isOneOf(["D", "A"]):
-                            # link gender and number of the noun to the determiners and adjectives
-                            # in English possessive determiner should not depend on the noun but on the "owner"
-                            if self.isFr() or not e.isA("D") or "own" not in e.props:
-                                e.peng = self.peng
-            # set agreement between the subject of a subordinate or the object of a subordinate
-            pro = self.getFromPath([["S", "SP"], "Pro"])
-            if pro is not None:
-                v = pro.parentConst.getFromPath(["VP", "V"])
-                if v is not None:
-                    if pro.lemma in ["qui", "who"]:  # agrees with self NP
-                        v.peng = self.peng
-                    elif self.isFr() and pro.lemma == "que":
-                        # in French past participle can agree with a cod appearing before... keep that info just in case
-                        v.cod = self
+            if hasattr(self.elements[headIndex],"peng"):
+                # must check because getHeadIndex returns the first element when it does not find anything appropriate
+                self.peng = self.elements[headIndex].peng
+                for i in range(0, len(self.elements)):
+                    if i != headIndex:
+                        e = self.elements[i]
+                        if hasattr(self, "peng"):  # do not try to modify if current peng does not exist e.g. Q
+                            if e.isA("NO") and i < headIndex:  # NO must appear before the N for agreement
+                                self.peng["n"] = e.grammaticalNumber()
+                                # gender agreement between a French number and subject
+                                e.peng["g"] = self.peng["g"]
+                            elif e.isOneOf(["D", "A"]):
+                                # link gender and number of the noun to the determiners and adjectives
+                                # in English possessive determiner should not depend on the noun but on the "owner"
+                                if self.isFr() or not (e.isA("D") and "own" in e.props):
+                                    e.peng = self.peng
+                            elif e.isA("CP"): # check for a coordination of adjectives and numbers
+                                for el in e.elements:
+                                    if el.isOneOf(["A","NO"]):
+                                        el.peng=self.peng
+                # set agreement between the subject of a subordinate or the object of a subordinate
+                pro = self.getFromPath([["S", "SP"], "Pro"])
+                if pro is not None:
+                    v = pro.parentConst.getFromPath(["VP", "V"])
+                    if v is not None:
+                        if pro.lemma in ["qui", "who", "which", "that"]:  # agrees with self NP
+                            v.peng = self.peng
+                            self.linkAttributes(v,self.getFromPath([["VP"],["CP"]]),self)
+                        elif self.isFr() and pro.lemma == "que":
+                            # in French past participle can agree with a cod appearing before... keep that info just in case
+                            v.cod = self
         elif self.isA("VP"):
             headIndex = self.getHeadIndex("VP")  # head is the first internal V
             self.peng = self.elements[headIndex].peng
@@ -207,22 +256,7 @@ class Phrase(Constituent):
                 vpv = self.linkPengWithSubject("VP", "V", subject)
                 if vpv is not None:
                     self.taux = vpv.taux
-                    if self.isFr() and vpv.lemma in ["être", "paraître", "sembler", "devenir", "rester"]:
-                        # check for a French attribute of copula verb
-                        # with an adjective
-                        attribute = vpv.parentConst.linkPengWithSubject("AP", "A", subject)
-                        if attribute is None:
-                            elems = vpv.parentConst.elements
-                            try:
-                                vpvIdx = elems.index(vpv)
-                                for i in range(vpvIdx + 1, len(elems)):
-                                    pp = elems[i]
-                                    if pp.isA("V") and pp.getProp("t") == "pp":
-                                        pp.peng = subject.peng
-                                        break
-                            except ValueError:
-                                self.error("linkProperties    : verb not found")
-
+                    self.linkAttributes(vpv,self.getFromPath([["VP"],["CP"]]),subject)
                 else:
                     # check for a coordination of verbs that share the subject
                     cvs = self.getFromPath(["CP", "VP"])
@@ -239,12 +273,12 @@ class Phrase(Constituent):
                             sppro = sp.getConst("Pro")
                             if sppro is not None and sppro.lemma == "que":
                                 v = sp.getFromPath([["VP", ""], "V"])
-                                if v != None:
+                                if v is not None:
                                     v.cod = cp
             else:
                 # finally, self generates too many spurious messages
                 # self.warning("no possible subject found")
-                self.peng = None
+                pass
         else:
             self.error("linkProperties    ,unimplemented type:" + self.constType)
         return self
@@ -253,11 +287,11 @@ class Phrase(Constituent):
         # do not link a subject pronoun at genitive
         if subject.isA("Pro") and "c" in subject.props and subject.props["c"] == "gen": return
         pt = self.getFromPath([phrase, terminal])
-        if pt != None:
+        if pt is not None:
             pt.parentConst.peng = pt.peng = subject.peng
         else:
             pt = self.getFromPath([terminal])
-            if pt != None:
+            if pt is not None:
                 pt.peng = subject.peng
         return pt
 
@@ -268,9 +302,9 @@ class Phrase(Constituent):
         return self.error("***: should never happen: setLemma: called on a Phrase")
 
     # find the index of a Constituent type (or one of the constituents) in the list of elements
-    def getIndex(self, constTypes):
+    def getIndex(self, constTypes,start=0):
         if isinstance(constTypes, str): constTypes = [constTypes]
-        for i in range(0, len(self.elements)):
+        for i in range(start, len(self.elements)):
             if self.elements[i].isOneOf(constTypes):
                 return i
         return -1
@@ -288,21 +322,22 @@ class Phrase(Constituent):
     #   set plural if one is plural or more than one combined with and
     #   set person to the minimum one encountered (i.e. 1st < 2nd < 3rd) mostly useful for French 
     def findGenderNumberPerson(self, andCombination):
-        g = "f"
+        g = None
         n = "s"
         pe = 3
         nb = 0
         for e in self.elements:
-            if e.isOneOf(["NP", "N", "Pro", "Q"]):
+            if e.isOneOf(["NP", "N", "Pro", "Q","NO"]):
                 nb += 1
                 propG = e.getProp("g")
-                if propG == "m" or propG == "x" or e.isA("q"): g = "m"  # masculine if gender is unspecified
+                if g is None and propG is not None: g = propG
+                if propG == "m": g = "m"  # masculine if one is specified
                 if e.getProp("n") == "p": n = "p"
                 propPe = e.getProp("pe")
-                if propPe != None and propPe < pe: pe = propPe
+                if propPe is not None and propPe < pe: pe = propPe
         if nb == 0:
             g = "m"
-        elif nb > 1 and n == "s" and andCombination:
+        elif nb > 1 and andCombination:
             n = "p"
         return {"g": g, "n": n, "pe": pe}
 
@@ -314,7 +349,7 @@ class Phrase(Constituent):
     def pronominalize_fr(self):
         npParent = self.parentConst
         pro = None
-        if npParent != None:
+        if npParent is not None:
             idxMe = npParent.elements.index(self)
             idxV = idxMe - 1  # search for the first verb before the NP
             while idxV >= 0 and not npParent.elements[idxV].isA("V"): idxV -= 1
@@ -334,7 +369,7 @@ class Phrase(Constituent):
             elif self.isA("PP"):  # is indirect complement
                 np = self.getFromPath([["NP", "Pro"]])  # either a NP or Pro within the PP
                 prep = self.getFromPath(["P"])
-                if prep != None and np != None:
+                if prep is not None and np is not None:
                     if prep.lemma == "à":
                         pro = np.getTonicPro("dat")
                     elif prep.lemma == "de":
@@ -372,7 +407,7 @@ class Phrase(Constituent):
     #  and does not need reorganisation of the sentence 
     def pronominalize_en(self):
         npParent = self.parentConst
-        if npParent != None:
+        if npParent is not None:
             idx = npParent.elements.index(self)
             if hasattr(npParent, "peng") and self.peng == npParent.peng:  # is subject
                 pro = self.getTonicPro("nom")
@@ -420,21 +455,18 @@ class Phrase(Constituent):
             vp = self
         else:
             vp = self.getConst("VP")
-            if vp != None:
-                if len(self.elements) > 0 and self.elements[0].isOneOf(["N", "NP", "Pro"]):
+            if vp is not None:
+                if len(self.elements) > 0 and self.elements[0].isOneOf(["N", "NP", "Pro","S"]):
                     subject = self.removeElement(0)
                     if subject.isA("Pro"):
-                        # as self pronoun will be preceded by "par" or "by", the "bare" tonic form is needed
-                        # to which the original pe, gender and number are added
-                        subject = subject.getTonicPro(None).pe(subject.getProp("pe")).g(subject.getProp("g")).n(
-                            subject.getProp("n"))
+                         subject = subject.getTonicPro()
                 else:
                     subject = None
             else:
                 return self.warn("not found", "VP", "contexte passif" if self.isFr() else "passive context")
         # remove object (first NP or Pro within VP) from elements
         newSubject = None
-        if vp != None:
+        if vp is not None:
             objIdx = vp.getIndex(["NP", "Pro"])
             if objIdx >= 0:
                 obj = vp.elements.pop(objIdx)
@@ -452,7 +484,11 @@ class Phrase(Constituent):
                 if self.isEn():
                     self.linkPengWithSubject("VP", "V", newSubject)
                 if subject is not None:  # insert subject where the object was
-                    vp.addElement(PP(P("par" if self.isFr() else "by", self.lang), subject), objIdx)
+                    if subject.isA("S"):  # take for granted that the S subject is a VP
+                        prep = "de" if self.isFr() else "to"
+                    else:
+                        prep = "par" if self.isFr() else "by"
+                    vp.addElement(PP(P(prep, self.lang), subject), objIdx)
             elif subject is not None:  # no object, but with a subject
                 # create a dummy subject with a "il"/"it" 
                 newSubject = Pro("lui" if self.isFr() else "it", self.lang).c("nom")
@@ -462,23 +498,29 @@ class Phrase(Constituent):
                 vp.peng = newSubject.peng
                 # add original subject after the verb to serve as an object
                 vpIdx = vp.getIndex("V")
-                vp.addElement(PP(P("par" if self.isFr() else "by", self.lang), subject), vpIdx + 1)
+                if subject.isA("S"):  # take for granted that the S subject is a VP
+                    prep = "de" if self.isFr() else "to"
+                    pos = None
+                else:
+                    prep = "par" if self.isFr() else "by"
+                    pos = vpIdx + 1
+                vp.addElement(PP(P(prep, self.lang), subject), pos)
             if self.isFr():
                 # do this only for French because in English this is done by processTyp_en
                 # change verbe to "être" auxiliary and make it agree with the newSubject
                 verbeIdx = vp.getIndex("V")
                 verbe = vp.removeElement(verbeIdx)
-                aux = V("être", "fr")
+                aux = V("avoir" if verbe.lemma == "être" else "être", "fr")
                 aux.parentConst = vp
                 aux.taux = verbe.taux
-                if newSubject != None:  # this can happen when a subject is Q
+                if newSubject is not None:  # this can happen when a subject is Q
                     aux.peng = newSubject.peng
                 aux.props = verbe.props
                 aux.pe(3)  # force person to be 3rd (number and tense will come from the pyrealb subject)
                 if vp.getProp("t") == "ip":
                     aux.t("s")  # set subjonctive present tense for an imperative
                 pp = V(verbe.lemma, "fr").t("pp")
-                if newSubject != None:  # self can happen when a subject is Q
+                if newSubject is not None:  # self can happen when a subject is Q
                     # make the past participle agree with the new subject
                     pp.setProp("g",newSubject.getProp("g"))
                     pp.setProp("n",newSubject.getProp("n"))
@@ -490,7 +532,7 @@ class Phrase(Constituent):
     # also deals with coordinated verbs
     def processVP(self, types, key, action):
         v = self.getFromPath(["CP", "VP"])
-        if v != None:  # possibly a coordination of verbs
+        if v is not None:  # possibly a coordination of verbs
             for e in self.getConst("CP").elements:
                 if e.isA("VP"):
                     e.processVP(types, key, action)
@@ -504,7 +546,7 @@ class Phrase(Constituent):
                 if idxVP >= 0:
                     vp = self.elements[idxVP]
                 else:
-                    self.warn("bad const for option", '.typ("' + key + ":" + str(val) + '")', self.constType, ["VP"])
+                    self.warn("not found", "VP",self.constType+'(...).typ("' + key + ":" + str(val) + '")')
                     return
             idxV = vp.getIndex("V")
             if idxV >= 0:
@@ -540,6 +582,7 @@ class Phrase(Constituent):
             for key in rules["verb_option"]["modalityVerb"]:
                 if key.startswith(mod):
                     v.setLemma(rules["verb_option"]["modalityVerb"][key])
+                    if hasattr(v,"cod"): del v.cod
                     break
             v.isMod=True
             i = idxV - 1
@@ -556,7 +599,7 @@ class Phrase(Constituent):
 
         self.processVP(types, "mod", mod)
 
-        def process_neg(vp, idxV, v, neg):
+        def process_neg(vp, _idxV, v, neg):
             if neg == True: neg = "pas"
             v.neg2 = neg  # HACK: to be used when conjugating at the realization time
             # while idxV>0 and vp.elements[idxV-1].isA("Pro"):idxV-=1
@@ -576,20 +619,19 @@ class Phrase(Constituent):
         auxils = []  # list of Aux followed by V
         affixes = []
         isFuture = False
-        if (t == "f"):
+        if t == "f" or t =="c":
             isFuture = True
-            t = "p"  # the auxiliary will be generated here so remove it from the V
+            t = "p" if t=="f" else "ps"# the auxiliary will be generated here so remove it from the V
         prog = "prog" in types and types["prog"] != False
         perf = "perf" in types and types["perf"] != False
         pas = "pas" in types and types["pas"] != False
         interro = types["int"] if "int" in types and types["int"] != False else None
         modality = types["mod"] if "mod" in types and types["mod"] != False else None
-        # compound = getRules()["compound"]
-        if modality != None:
+        if modality is not None:
             auxils.append(compound[modality]["aux"])
             affixes.append("b")
         elif isFuture:
-            # caution: future in English is done with the modal will, so another modal cannot be used
+            # caution: future and conditional in English are done with the modal will, so another modal cannot be used
             auxils.append(compound["future"]["aux"])
             affixes.append("b")
         if perf or prog or pas:
@@ -602,12 +644,13 @@ class Phrase(Constituent):
             if pas:
                 auxils.append(compound["passive"]["aux"])
                 affixes.append(compound["passive"]["participle"])
-        elif (interro != None and
+        elif (interro is not None and
               len(auxils) == 0 and v.lemma != "be" and v.lemma != "have"):
             # add auxiliary for interrogative if not already there
-            if interro != "wos" and interro != "was":
-                auxils.append("do")
-                affixes.append("b")
+            if interro not in ["wos","was","tag"]:
+                if t not in ["pp","pr","b-to","b"]: # do not add auxiliary for participle and infinitive
+                    auxils.append("do")
+                    affixes.append("b")
         auxils.append(v.lemma)
         # realise the first verb, modal or auxiliary
         # but make the difference between "have" as an auxiliary and "have" as a verb
@@ -615,9 +658,15 @@ class Phrase(Constituent):
         words = []
         # conjugate the first verb
         if neg:  # negate the first verb
-            if t == "pp" or t == "pr":  # special case for these tenses
+            if t in ["pp","pr","b-to","b"]:  # special case for these tenses
                 words.append(Adv("not", "en"))
+                if t == "b" : words.append(P("to","en"))
                 words.append(V(vAux, "en").t(t))
+            elif t == "ip" and v_peng["pe"] == 1 and v_peng["n"]=="p":
+                # very special case: insert "not" between "let's" and verb
+                words.append(Q("let's"))
+                words.append(Adv("not","en"))
+                words.append(V(vAux,"en").t("b"))
             elif vAux in negMod:
                 if vAux == "can" and t == "p":
                     words.append(Q("cannot"))
@@ -632,8 +681,7 @@ class Phrase(Constituent):
                 words.append(Adv("not", "en"))
                 if vAux != "do": words.append(V(vAux).t("b"))
         else:  # must only set necessary options, so that shared properties will work ok
-            newAux = V(vAux)
-            if not isFuture: newAux.t(t)
+            newAux = V(vAux).t(t)
             if v in negMod: newAux.pe(1)
             words.append(newAux)
         # recover the original agreement info and set it to the first pyrealb verb...
@@ -662,9 +710,10 @@ class Phrase(Constituent):
         if idxV >= 0:
             v = vp.elements[idxV]
             if "contr" in types and types["contr"] != False:
-                vp.contraction = True  # necessary because we want the negation to be contracted within the VP before the S or SP
+                vp.contraction = True  # necessary because we want the negation to be contracted
+                                       # within the VP before the S or SP
                 self.contraction = True
-            words=Phrase.affixHopping(vp.elements[idxV],vp.getProp("t"),getRules()["compound"],types)
+            words=Phrase.affixHopping(v,vp.getProp("t"),getRules()["compound"],types)
             # insert the content of the word array into vp.elements
             vp.removeElement(idxV)
             for i in range(0, len(words)):
@@ -679,7 +728,7 @@ class Phrase(Constituent):
             if idx >= 0: return (idx, self.elements)
         elif self.isOneOf(["S", "SP"]):
             cst = self.getConst(cst1)
-            if cst != None: return cst.getIdxCtx(cst1, cst2)
+            if cst is not None: return cst.getIdxCtx(cst1, cst2)
         return (None, None)
 
     def moveAuxToFront(self):
@@ -687,7 +736,8 @@ class Phrase(Constituent):
         if self.isEn():
             if self.isOneOf(["S", "SP"]):
                 (idx, vpElems) = self.getIdxCtx("VP", "V")
-                if idx != None:
+                if idx is not None and self.getProp("t") not in ["pp","pr","b-to"]:
+                    # do not move when tense is participle or infinitive
                     v = vpElems.pop(0)  # remove first V
                     # check if V is followed by a negation, if so move it also
                     if len(vpElems) > 0 and vpElems[0].isA("Adv") and vpElems[0].lemma == "not":
@@ -713,12 +763,13 @@ class Phrase(Constituent):
             else:
                 pro = Pro("moi", "fr").g(subj.getProp("g")).n(subj.getProp("n")).pe(3).c("nom")  # create a pronoun
             (idx, vpElems) = self.getIdxCtx("VP", "V")
-            if idx != None:
+            if idx is not None:
                 v = vpElems[idx]
                 v.parentConst.addElement(pro, idx + 1)  # add pronoun after verb
                 v.lier()  # add - after verb
 
-    def processInt(self, int_):
+    def processInt(self, types):
+        int_=types["int"]
         sentenceTypeInt = getRules()["sentence_type"]["int"]
         intPrefix = sentenceTypeInt["prefix"]
         prefix = None
@@ -733,30 +784,34 @@ class Phrase(Constituent):
         elif int_ in ["wos", "was"]:  # remove subject (first NP,N, Pro or SP)
             if self.isOneOf(["S", "SP", "VP"]):
                 subjIdx = self.getIndex(["NP", "N", "Pro", "SP"])
-                if subjIdx != None:
+                if subjIdx is not None:
                     vbIdx = self.getIndex(["VP", "V"])
-                    if vbIdx != None and subjIdx < vbIdx:  # subject should be before the verb
-                        # make sure that the verb at the third-person singular,
+                    if vbIdx is not None and subjIdx < vbIdx:  # subject should be before the verb
+                        # make sure that the verb at the third-person
                         # because now the subject has been removed
                         v = self.elements[vbIdx]
-                        v.setProp("n", "s")
                         v.setProp("pe", 3)
                         del self.elements[subjIdx]
             prefix = intPrefix[int_]
         elif int_ in ["wod", "wad"]:  # remove direct object (first NP,N,Pro or SP in the first VP)
+            cmp=None
             if self.isOneOf(["S", "SP", "VP"]):
                 idx, obj = self.getIdxCtx("VP", ["NP", "N", "Pro", "SP"])
-                if idx != None:
+                if idx is not None:
                     del obj[idx]
                 elif self.isFr():  # check for passive subject starting with par
                     idx, ppElems = self.getIdxCtx("VP", "PP")
-                    if idx != None:
+                    if idx is not None:
                         pp = ppElems[idx].getConst("P")
-                        if pp != None and pp.lemma == "par":
-                            ppElems[0].parentConst.removeElement(idx)  # remove the passive subject
+                        if pp is not None and pp.lemma == "par":
+                            cmp = ppElems[0].parentConst.removeElement(idx)  # remove the passive subject
                         else:
                             pp = None
-                prefix = intPrefix[int_]
+                if self.isEn() and int == "wod" and cmp is not None and cmp.getProp("g") in ["m","f"]:
+                    # human direct object
+                    prefix = "whom"
+                else:
+                    prefix = intPrefix[int_]
                 if self.isEn():
                     self.moveAuxToFront()
                 else:
@@ -765,7 +820,7 @@ class Phrase(Constituent):
             if self.isOneOf(["S", "SP", "VP"]):
                 idx, ppElems = self.getIdxCtx("VP", "PP")
                 prefix = intPrefix[int_]  # get default prefix
-                if idx != None:
+                if idx is not None:
                     # try to find a more appropriate prefix by looking at preposition in the structure
                     prep = ppElems[idx].elements[0]
                     if prep.isA("P"):
@@ -786,11 +841,69 @@ class Phrase(Constituent):
                     self.moveAuxToFront()
                 else:
                     self.invertSubject()
+        elif int_=="tag":
+            # according to Antidote: Syntax Guide - Question tag
+            # Question tags are short questions added after affirmations to ask for verification
+            if self.isOneOf(["S", "SP", "VP"]):
+                if self.isFr(): # in French really simple, add "n'est-ce pas"
+                    self.a(", n'est-ce pas")
+                else: # in English, sources: https://www.anglaisfacile.com/exercices/exercice-anglais-2/exercice-anglais-95625.php
+                      # must find  and pronoun and conjugate the auxiliary
+                    currV=self.getFromPath(["VP","V"])
+                    if currV is not None:
+                        if "mod" in types and types["mod"] != False :
+                            aux=getRules()["compound"][types["mod"]]["aux"]
+                        else:
+                            if currV.lemma in ["have","be","can","will","shall","may","must"]:
+                                aux=currV.lemma
+                            else:
+                                aux="do"
+                    neg="neg" in types and types["neg"]==True
+                    pe = currV.getProp("pe")
+                    t  = currV.getProp("t")
+                    n  = currV.getProp("n")
+                    g  = currV.getProp("g")
+                    pro = Pro("I").pe(pe).n(n).g(g)  # get default pronoun
+                    subjIdx = self.getIndex(["NP","N","Pro","SP"])
+                    if subjIdx >= 0:
+                        vbIdx=self.getIndex(["VP","V"])
+                        if vbIdx >= 0 and subjIdx<vbIdx: # subject should be before the verb
+                            subj=self.elements[subjIdx]
+                            if subj.isA("Pro"):
+                                if subj.getProp("pe")==1 and aux=="be" and t=="p" and not neg:
+                                    # very special case : I am => aren't I
+                                    pe=2
+                                elif subj.lemma in ["this","that","nothing"]:
+                                    pro=Pro("I","en").g("n") # it
+                                elif subj.lemma in ["somebody","anybody","nobody","everybody",
+                                                     "someone","anyone","everyone"]:
+                                    pro=Pro("I","en").n("p") # they
+                                    if subj.lemma=="nobody":neg=True
+                                else:
+                                    pro=subj.clone()
+                            else:
+                                pro=subj.clone().pro()
+                    else: # no subject, but check if the verb is imperative
+                        if t == "ip":
+                            if aux =="do": aux = "will" # change aux when the aux is default
+                            pro = Pro("I","en").pe(2).n(n).g(g)
+                    # check for negative adverb
+                    adv = currV.parentConst.getConst("Adv")
+                    if adv is not None and adv.lemma in ["hardly","scarcely","never","seldom"]:
+                        neg=True
+                    currV.parentConst.a(",")
+                    #   this is a nice illustration of jsRealB using itself for realization
+                    if aux=="have" and not neg:
+                        # special case because it should be realized as "have not" instead of "does not have"
+                        self.addElement(VP(V("have").t(t).pe(pe).n(n),Adv("not"),pro).typ({"contr":True}))
+                    else: # use jsRealB itself for realizing the tag by adding a new VP
+                        self.addElement(VP(V(aux).t(t).pe(pe).n(n),pro).typ({"neg":not neg,"contr":True}))
+            prefix = intPrefix[int_]
         else:
             self.warn("not implemented", "int:" + int_)
         if self.isFr() or int_ != "yon":  # add the interrogative prefix
             self.addElement(Q(prefix), 0)
-            if pp != None:  # add "par" in front of some French passive interrogative
+            if pp is not None:  # add "par" in front of some French passive interrogative
                 self.addElement(pp, 0)
                 if int_ == "wad":  # replace "que" by "quoi" for French passive wad
                     self.elements[1].lemma = "quoi"
@@ -806,14 +919,12 @@ class Phrase(Constituent):
         else:
             self.processTyp_en(types)
         if "int" in types and types["int"] != False:
-            self.processInt(types["int"])
+            self.processInt(types)
         if "exc" in types and types["exc"] == True:
             self.a(getRules()["sentence_type"]["exc"]["punctuation"], True)
         return self
 
-
     ######### Realization
-
     #  special case of realization of a cp for which the gender and number must be computed
     #    at realization time...
 
@@ -832,7 +943,7 @@ class Phrase(Constituent):
             res = elems[0].real()
             self.setProp("g", elems[0].getProp("g"))
             self.setProp("n", elems[0].getProp("n"))
-            self.setProp("pe", elems[0].getProp("pe") if elems[0].getProp("pe") != None else 3)
+            self.setProp("pe", elems[0].getProp("pe") if elems[0].getProp("pe") is not None else 3)
             return self.doFormat(res)  # process format for the CP
         for j in range(0, last):  # insert comma after each element
             ej = elems[j]
@@ -850,7 +961,8 @@ class Phrase(Constituent):
             c = self.elements[idxC]
             _and = "et" if self.isFr() else "and"
             gn = self.findGenderNumberPerson(c.lemma == _and)
-            self.setProp("g", gn["g"])
+            if gn["g"] is not None:
+                self.setProp("g", gn["g"])
             self.setProp("n", gn["n"])
             self.setProp("pe", gn["pe"])
             # for the pronoun, we must override its existing properties...
@@ -859,7 +971,7 @@ class Phrase(Constituent):
                 self.pronoun.props["g"] = gn["g"]
                 self.pronoun.props["n"] = gn["n"]
                 self.pronoun.props["pe"] = gn["pe"]
-        return res
+        return self.doFormat(res)
 
     def real(self):
         res = []
@@ -878,6 +990,8 @@ class Phrase(Constituent):
                 else:
                     r = e.real()
                 res.extend(r)
+            if self.isA("VP"):
+                self.checkAdverbPos(res)
         return self.doFormat(res)
 
     # recreate a jsRealB expression
@@ -908,11 +1022,12 @@ class Phrase(Constituent):
                 args = [fromJSON(e, lang) for e in json["elements"]]
                 return Phrase(constType, args, lang).setJSONprops(json)
             else:
-                return self.warn("user-warning","Phrase.fromJSON elements should be a list:" + str(json["elements"]))
+                return self.warn(Q(""),"user-warning","Phrase.fromJSON elements should be a list:" + str(json["elements"]))
         else:
-            return self.warn("user-warning","Phrase.fromJSON: no elements found in " + str(json))
+            return self.warn(Q(""),"user-warning","Phrase.fromJSON: no elements found in " + str(json))
 
     # create a Dependent version of a Phrase
+    #   CAUTION:  this is useful for "common cases", this is not "foolproof" in all cases
     def toDependent(self,depName=None):
         from .Dependent import Dependent
 
@@ -1046,4 +1161,8 @@ class S(Phrase):
 
 class SP(Phrase):
     def __init__(self, *elems):
+        """
+
+        :rtype: object
+        """
         super().__init__("SP", elems)
