@@ -1,8 +1,8 @@
-from .Constituent import Constituent,quoteOOV
+from .Constituent import Constituent,quoteOOV, deprels
 from .Number import enToutesLettres, ordinal
 from .Lexicon import getLexicon,currentLanguage,getLemma,getRules
 
-import datetime, sys,re,copy
+import datetime, re
 
 class Terminal(Constituent):
     def __init__(self, terminalType,lemma,lang=None):
@@ -71,7 +71,7 @@ class Terminal(Constituent):
                     self.value=int(self.lemma)
                 except ValueError:
                     self.value=float(self.lemma)
-                self.nbDecimals=self.lemma[::-1].find(".")
+                self.nbDecimals=str(self.lemma)[::-1].find(".")
             else:
                 self.value=lemma
                 self.nbDecimals=str(lemma)[::-1].find(".")
@@ -89,7 +89,7 @@ class Terminal(Constituent):
                 self.tab=None 
                 self.realization=f"[[{lemma}]]"
                 return self.warn("not in lexicon",self.lang)
-                if quoteOOV:
+                if quoteOOV: # not currently used
                     self.lemma=str(lemma)
                     self.realization=self.lemma
                     self.constType="Q"
@@ -97,14 +97,16 @@ class Terminal(Constituent):
                 if terminalType not in lexInfo:
                     self.tab=None 
                     self.realization=f"[[{lemma}]]"
-                    return self.warn("not in lexicon",self.lang,lexInfo.keys())
-                    if quoteOOV:
+                    otherPOS=list(lexInfo.keys())
+                    if "basic" in otherPOS:otherPOS.remove("basic")
+                    return self.warn("not in lexicon",self.lang,otherPOS)
+                    if quoteOOV: # not currently used
                         self.lemma=str(lemma)
                         self.realization=self.lemma
                         self.constType="Q"
                 else:
                     lexInfo=lexInfo[terminalType]
-                    rules=getRules()
+                    rules=getRules(self.lang)
                     for (key,info) in lexInfo.items():
                         if key=="tab": # save table number and compute stem
                             ending=None 
@@ -114,7 +116,8 @@ class Terminal(Constituent):
                                     declension=rules["declension"][self.tab]
                                     ending=declension["ending"]
                                     if terminalType=="Pro":
-                                        #  set person for Pro when different than 3 (i.e. all elements of declension are the same)
+                                        #  set person for Pro when different than 3 (i.e. all elements of
+                                        #  declension are the same)
                                         dd=declension["declension"]
                                         if "pe" in dd[0]:
                                             pe=dd[0]["pe"]
@@ -130,7 +133,7 @@ class Terminal(Constituent):
                                 else:
                                     ending=""
                                     self.warn("bad lexicon table",lemma,ending)
-                            if ending!=None and self.lemma.endswith(ending):
+                            if ending is not None and self.lemma.endswith(ending):
                                 self.stem=self.lemma if ending=="" else self.lemma[:-len(ending)]
                             else:
                                 self.tab=None 
@@ -138,7 +141,7 @@ class Terminal(Constituent):
                                     self.warn("bad lexicon table",lemma,ending)
                         else:
                             # if isinstance(info,list) and len(info)==1:info=info[0]
-                            self.setProp(key,info)
+                            self.setProp(key,info,True)
         else:
             self.warn("not implemented",terminalType)
     
@@ -152,7 +155,7 @@ class Terminal(Constituent):
             # according to http:#bdl.oqlf.gouv.qc.ca/bdl/gabarit_bdl.asp?id=1582
             return "s" if -2<number<2 else "p"
         else:
-            # according to https:#www.chicagomanualofstyle.org/book/ed17/part2/ch09/psec019.html
+            # according to https://www.chicagomanualofstyle.org/book/ed17/part2/ch09/psec019.html
             #   any number other than 1 is plural... 
             # even 1.0 but this case is not handled here because nbDecimal(1.0)=>0
             return "s" if abs(number)==1 and self.nbDecimals==0 else "p"
@@ -189,55 +192,65 @@ class Terminal(Constituent):
         return bestMatch[1]                 
                                      
     def decline(self,setPerson):
-        rules=getRules()
+        rules=getRules(self.lang)
         declension=rules["declension"][self.tab]["declension"]
         stem=self.stem
         res=None 
         if self.isOneOf(["A","Adv"]):
             if self.isFr():
+                # special case of French adjectives or adv, they can have more than one token
                 g = self.getProp("g")
                 n = self.getProp("n")
                 ending=self.bestMatch("déclinaison d'adjectif", declension,{"g":g,"n":n})
                 if ending is None:
-                    return f"[[{self.lemma}]]"
-                res=stem+ending
+                    return [self.morphoError("decline [fr]: A", {"g":g,"n":n})]
                 f = self.getProp("f") # comparatif d'adjectif
-                if f!=None and f != False:
+                if f is not None and f != False:
                     specialFRcomp={"bon":"meilleur","mauvais":"pire"}
+                    res = []
                     if f=="co":
                         if self.lemma in specialFRcomp:
-                            return A(specialFRcomp[self.lemma]).g(g).n(n).realize()
+                            self.insertReal(res,A(specialFRcomp[self.lemma]).g(g).n(n))
                         else:
-                            return "plus "+res
-                    if f=="su":
-                        art=D("le").g(g).n(n).realize()+" "
+                            self.insertReal(res,Adv("plus"))
+                            self.insertReal(res,A(self.lemma).g(g).n(n))
+                    elif f=="su":
+                        self.insertReal(res,D("le").g(g).n(n))
                         if self.lemma in specialFRcomp:
-                            return art+A(specialFRcomp[self.lemma]).g(g).n(n).realize()
+                            self.insertReal(res,A(specialFRcomp[self.lemma]).g(g).n(n))
                         else:
-                            return art+"plus "+res
-            else: 
-                # English adjective/adverbs are invariable but they can have comparative
-                res=self.lemma
+                            self.insertReal(res,Adv("plus"))
+                            self.insertReal(res,A(self.lemma).g(g).n(n))
+                    return res
+                else:
+                    self.realization = self.stem+ending
+                    return [self]
+            else:
+                # English adjective/adverbs are invariable but they can have comparative, thus return more than one token
+                self.realization = self.lemma
                 f = self.getProp("f")
-                if f!=None and f!=False:
+                if f is not None and f!=False:
                     if self.tab=="a1":
-                        res=("more " if f=="co" else "most ")+res
+                        comp=A("more " if f=="co" else "most ")
+                        comp.realization = comp.lemma
+                        return [comp,self]
                     else:
                         if self.tab=="b1": # adverb without comparative/superlative, try the adjective table
                             adjAdv=getLemma(self.lemma)
                             if "A" in adjAdv:
-                                declension=rules["declension"][adjAdv["tab"]]["declension"]
-                                ending=rules["declension"][adjAdv["tab"]]["ending"]
+                                declension=rules["declension"][adjAdv["A"]["tab"]]["declension"]
+                                ending=rules["declension"][adjAdv["A"]["tab"]]["ending"]
                                 stem=stem[0:-len(ending)]
                             else:  # adverb without adjective
-                                return res
+                                return [self]
                         # look in the adjective declension table
                         ending=self.bestMatch("adjective declension",declension,{"f":f})
                         if ending is None:
-                            return f"[[{self.lemma}]]"
-                        res=stem+ending
+                            return [self.morphoError("decline [en]: A", {"f":f})]
+                        self.realization = self.stem + ending
+                        return [self]
         elif len(declension)==1: # no declension
-            res=stem+declension[0]["val"]
+            self.realization = self.stem+declension[0]["val"]
         else: # for N,D,Pro
             g=self.getProp("g")
             if self.isOneOf(["D","N"]) and g is None:g="m"
@@ -252,7 +265,7 @@ class Terminal(Constituent):
                 keyVals["own"]=self.props["own"]
             if self.isA("Pro"):
                 c  = self.props["c"] if "c" in self.props else None
-                if c!= None:
+                if c is not None:
                     if self.isFr() and c=="gen": # genitive cannot be used in French
                         self.warn("ignored value for option","c",c)
                     elif self.isEn() and c=="refl": # reflechi cannot be used in English
@@ -260,13 +273,13 @@ class Terminal(Constituent):
                     else:
                         keyVals["c"]=c
                 tn = self.props["tn"] if "tn" in self.props else None
-                if tn != None:
-                    if c!= None:
+                if tn  is not None:
+                    if c is not None:
                         self.warn("both tonic and clitic")
                     else:
                         keyVals["tn"]=tn
-                if c != None or tn != None:
-                    if ((self.isFr() and self.lemma=="moi")) or  ((self.isEn() and self.lemma=="me")):
+                if c  is not None or tn  is not None:
+                    if (self.isFr() and self.lemma=="moi") or  (self.isEn() and self.lemma=="me"):
                         # HACK:remove defaults from pronoun such as "moi" in French and "me" in English
                         #      because their definition is special in order to try to keep some upward compatibility
                         #      with the original way of specifying the pronouns
@@ -287,20 +300,18 @@ class Terminal(Constituent):
                     if self.lemma!="on": keyVals["tn"]=""
             ending=self.bestMatch("déclinaison" if self.isFr() else "declension",declension,keyVals)
             if ending is None:
-                return f"[[{self.lemma}]]"
+                return [self.morphoError("decline [en]: N,D Pro", {"g": g, "n": n, "pe":pe})]
             if self.isFr() and self.isA("N"):
                 # check is French noun gender specified corresponds to the one given in the lexicon
-                lexiconG=getLexicon()[self.lemma]["N"]
+                lexiconG=getLexicon("fr")[self.lemma]["N"]
                 if "g" not in lexiconG:
-                    self.morphoError("absent du lexique",{"g":g,"n":n})
-                    return f"[[{self.lemma}]]"
+                    return [self.morphoError("genre absent du lexique",{"g":g,"n":n})]
                 lexiconG=lexiconG["g"]
                 if lexiconG != "x" and lexiconG != g:
-                    self.morphoError(
-                        "genre différent de celui du lexique",{"g":g, "lexique":lexiconG})
-                    return f"[[{self.lemma}]]"
-            res=stem+ending
-        return res
+                    return [self.morphoError(
+                        "genre différent de celui du lexique",{"g":g, "lexique":lexiconG})]
+            self.realization = self.stem+ending
+        return [self]
 
     def removeNextConstInSentence(self):
         parentElems=self.parentConst.elements
@@ -325,7 +336,7 @@ class Terminal(Constituent):
         if isinstance(newTerminal,Terminal):
             newTerminal.parentConst=self.parentConst
             newTerminal.realize()
-            if position==None:
+            if position is None:
                 terms.append(newTerminal)
             else:
                 terms.insert(position,newTerminal)
@@ -336,13 +347,14 @@ class Terminal(Constituent):
     def isReflexive(self):
         if not self.isA("V"):
             return self.error("isReflexive() should be called only for a verb,  not a "+self.constType)
-        if hasattr(self,"ignoreRefl"):return False; #HACK: this might be set in Phrase.processTyp_fr when dealing with "progressive"
+        if hasattr(self,"ignoreRefl"):return False # HACK: this might be set in Phrase.processTyp_fr when
+                                                   # dealing with "progressive"
         pat=self.getProp("pat")
-        if pat!=None and len(pat)==1 and pat[0]=="réfl": return True # essentiellement réflexif
+        if pat is not None and len(pat)==1 and pat[0]=="réfl": return True # essentiellement réflexif
         # check for "refl" typ (only called for V): Terminal.conjugate_fr
         pc=self.parentConst
         while pc is not None:
-            if pc.isOneOf(["VP","SP","S"]):
+            if pc.isOneOf(["VP","SP","S",*deprels]):
                 if "typ" in pc.props:
                     typs=pc.props["typ"]
                     if "refl" in typs and typs["refl"]==True:
@@ -361,11 +373,10 @@ class Terminal(Constituent):
         g = self.getProp("g")
         n = self.getProp("n")
         t = self.getProp("t")
-        neg=None 
         if self.tab is None:
             return [self.morphoError("conjugate_fr:tab",{"pe":pe,"n":n,"t":t})] 
-        if t in ["pc","pq","cp","fa","spa","spq"]: # temps composés
-            tempsAux={"pc":"p","pq":"i","cp":"c","fa":"f","spa":"s","spq":"si"}[t]
+        if t in ["pc","pq","cp","pa","fa","spa","spq","bp"]: # temps composés
+            tempsAux={"pc":"p","pq":"i","cp":"c","pa":"ps","fa":"f","spa":"s","spq":"si","bp":"b"}[t]
             aux=V("avoir","fr")
             aux.parentConst=self.parentConst
             aux.peng=self.peng
@@ -376,12 +387,14 @@ class Terminal(Constituent):
             elif aux.taux["aux"]=="êt":
                 aux.setLemma("être")
             else: # auxiliary "avoir"
-                # check the gender and number of a cod appearing before the verb to do proper agreement of its part participle
-                g="m"
-                n="s"
-                if hasattr(self,"cod"):
-                    g=self.cod.getProp("g")
-                    n=self.cod.getProp("n")
+                # check the gender and number of a cod appearing before the verb to do proper agreement
+                # of its part participle, except when the verb is "être" which will always agree
+                if self.lemma != "être":
+                    g="m"
+                    n="s"
+                    if hasattr(self,"cod"):
+                        g=self.cod.getProp("g")
+                        n=self.cod.getProp("n")
             aux.taux["t"]=tempsAux
             aux.realization=aux.realize()
             # change this verb to pp
@@ -397,7 +410,7 @@ class Terminal(Constituent):
                 del self.props["lier"] # delete it from the verb
                 # HACK: check if the verb was lié to a nominative pronoun (e.g. subject inversion for a question)
                 myParent=self.parentConst
-                if myParent != None:
+                if myParent is not None:
                     if hasattr(myParent,"elements"): # when dealing with a Phrase
                         myParentElems=myParent.elements
                         idxMe=myParentElems.index(self)
@@ -406,23 +419,26 @@ class Terminal(Constituent):
                             next=myParentElems[idxNext]
                             if next.isA("Pro"):
                                 thePro=myParentElems.pop(idxNext) # remove next pro from parent
-                                thePro.realization=thePro.realize() # insert its realization after the auxiliary and before the verb
+                                thePro.realization=thePro.realize() # insert its realization after the auxiliary
+                                                                    # and before the verb
                                 return [aux,thePro,self]
                     elif hasattr(myParent,"dependents"): # when dealing with a Dependent
                         proIndex=myParent.findIndex(lambda d:d.terminal.isA("Pro"))
                         if proIndex>=0:
                             thePro=myParent.removeDependent(proIndex).terminal  # remove Pro from Parent
-                            thePro2=Pro(thePro.lemma)             # as the original Pro is already realized in the output list, we must hack
+                            thePro2=Pro(thePro.lemma)             # as the original Pro is already realized in the
+                                                                  # output list, we must hack
                             thePro2.props=thePro.props            # and we cannot use clone because of environ
                             thePro2.peng=thePro.peng
-                            thePro2.realization=thePro2.realize() # insert its realization after the auxiliary and before the verb
+                            thePro2.realization=thePro2.realize() # insert its realization after the auxiliary
+                                                                  # and before the verb
                             thePro.realization=""                 # set original Pro realization to nothing
                             return [aux,thePro2,self]
                     else:
                         self.error("Terminal.conjugate_fr:: Strange parent:" + type(myParent).__name__)
             return [aux,self]
         else: # simple tense
-            conjugationTable=getRules()["conjugation"][self.tab]
+            conjugationTable=getRules(self.lang)["conjugation"][self.tab]
             if "t" in conjugationTable and t in conjugationTable["t"]:
                 conjugation=conjugationTable["t"]
                 if t in ["p","i","f","ps","c","s","si"]:
@@ -444,28 +460,27 @@ class Terminal(Constituent):
                     else:
                         self.realization=self.stem+term
                     res=[self]
-                    if self.isReflexive() and self.parentConst==None:
+                    if self.isReflexive() and self.parentConst is None:
                         self.lier()
                         self.insertReal(res,Pro("moi","fr").tn("").pe(pe).n(n).g(g))
                     return res
                 elif t in ["b","pr","pp"]:
                     self.realization=self.stem+conjugation[t]
                     res=[self]
-                    if t!="pp" and self.isReflexive() and self.parentConst==None:
+                    if t!="pp" and self.isReflexive() and self.parentConst is None:
                         self.insertReal(res, Pro("moi","fr").c("refl").pe(pe).n(n).g(g), 0)
                     if t=="pp" and self.realization!="été": # HACK: frequent case of être that does not change
                         g=self.getProp("g")
-                        if g=="x":g="m"
+                        if g=="x" or g=="n":g="m"   # neutre peut arriver si le sujet est en anglais
                         n=self.getProp("n")
                         if n=="x":n="s"
-                        term={"ms":"","mp":"s","fs":"e","fp":"es"}[g+n]
-                        self.realization+=term
-                    # neg=self.neg2 if hasattr(self,"neg2") else None
-                    # if neg!=None and neg!="":
-                    #     if t=="b" or t=="pp":
-                    #         self.insertReal(res,Adv(neg,"fr"),0)
-                    #     else:
-                    #         self.insertReal(res,Adv(neg,"fr"))
+                        if (g+n)=="mp" and self.realization.endswith("s"):
+                            pass    # pas d'ajout de s au masculin pluriel si la réalisation termine en s
+                        else:
+                            if (g+n)!="ms" and self.realization.endswith("û"): #changer "dû" en "du" sauf pour masc sing
+                                self.realization = self.realization[:-1]+"u"+{"ms":"","mp":"s","fs":"e","fp":"es"}[g+n]
+                            else:
+                                self.realization+={"ms":"","mp":"s","fs":"e","fp":"es"}[g+n]
                     return res
                 else:
                     return [self.morphoError("conjugate_fr",{"pe":pe,"n":n,"t":t})]
@@ -473,7 +488,8 @@ class Terminal(Constituent):
                 return [self.morphoError("conjugate_fr",{"pe":pe,"n":n,"t":t})]
     
     def conjugate_en(self):
-        pe=int(self.getProp("pe"))
+        pe=self.getProp("pe")
+        pe = 3 if pe is None else int(pe)
         n = self.getProp("n")
         t = self.getProp("t")
         if self.tab is None:
@@ -481,7 +497,7 @@ class Terminal(Constituent):
         # subjonctive present is like present except that it does not end in s at 3rd person
         # subjonctive past is like simple past
         t1 = "p" if t=="s" else ("ps" if t=="si" else t)
-        conjugationTable=getRules()["conjugation"][self.tab]
+        conjugationTable=getRules(self.lang)["conjugation"][self.tab]
         res=[self]
         if "t" in conjugationTable and t1 in conjugationTable["t"]:
             conjugation=conjugationTable["t"][t1]
@@ -500,11 +516,19 @@ class Terminal(Constituent):
             elif t in ["b","pp","pr"]:
                 self.realization=self.stem+conjugation
                 return [self]
-        elif t=="f":
+        elif t == "f":
             self.realization=self.lemma
-            self.insertReal(res,Q("will"),0)
-        elif t=="ip":
+            self.insertReal(res,V("will"),0)
+        elif t == "c":
             self.realization=self.lemma
+            self.insertReal(res,V("will").t("ps"),0)
+        elif t == "b-to":
+            self.realization = self.lemma
+            self.insertReal(res,P("to"),0)
+        elif t == "ip":
+            self.realization=self.lemma
+            if pe == 1 and n == "p":
+                self.insertReal(res,Q("let's"),0)
         else:
             return [self.morphoError("conjugate_en:tab",{"pe":pe,"n":n,"t":t})]
         return res 
@@ -514,7 +538,7 @@ class Terminal(Constituent):
         return self.conjugate_fr() if self.isFr() else self.conjugate_en()       
 
     ### Number 
-    ###   implementation differes from the javascript one using format 
+    ###   implementation differes from the JavaScript one using the Python format function
     ###    we avoid using locale...
     def numberFormatter(self,maxPrecision=None):
         if isinstance(self.value,int):
@@ -522,7 +546,7 @@ class Terminal(Constituent):
         else:
             precision = 2 if maxPrecision  is None else maxPrecision
         res=("{:,."+str(precision)+"f}").format(self.value)
-        symbol = getRules()["number"]["symbol"]
+        symbol = getRules(self.lang)["number"]["symbol"]
         if symbol["group"]!=",":res=res.replace(",",symbol["group"])
         if symbol["decimal"]!=".":res=res.replace(".",symbol["decimal"]) 
         return res
@@ -544,8 +568,8 @@ class Terminal(Constituent):
     
     ### Date
     def dateFormat(self,dateObj,dOpts):
-        fmtRE=re.compile(r"(.*?)\[(.+?)\]|(.+$)")
-        dateRule = getRules()["date"]
+        fmtRE=re.compile(r"(.*?)\[(.+?)]|(.+$)")
+        dateRule = getRules(self.lang)["date"]
         fmts=dateRule["format"]["natural" if dOpts["nat"] else "non_natural"]
 
         def interpret(fields):
@@ -580,46 +604,51 @@ class Terminal(Constituent):
         if isinstance(dOpts["rtime"],datetime.datetime):
             relativeDate = dateRule["format"]["relative_time"]
             # find the number of days of difference between relDay and the current date
-            diffDays=(dateObj-dOpts["rtime"]).days  # use datetime arithmetic to create timedelta
+            # use the "proleptic Gregorian ordinal number" to get the right number of days
+            diffDays=dateObj.toordinal()-dOpts["rtime"].toordinal()
             if str(diffDays) in relativeDate: # within a week before or after
-                return relativeDate[str(diffDays)].replace("[l]",
+                dateS = relativeDate[str(diffDays)].replace("[l]",
                             dateRule["text"]["weekday"][(dateObj.weekday()+1)%7])
-            ## more than a week
-            sign="-" if diffDays<0 else "+"
-            return relativeDate[sign].replace("[x]",str(abs(diffDays)))
-        
-        ## process date fields
-        dateS=interpret("-".join(field for field in ["year","month","date","day"] if dOpts[field]))
+            else:
+                ## more than a week
+                sign="-" if diffDays<0 else "+"
+                dateS = relativeDate[sign].replace("[x]",str(abs(diffDays)))
+        else:
+            ## process date fields
+            dateS = interpret("-".join(field for field in ["year", "month", "date", "day"] if dOpts[field]))
+
         timeS=interpret(":".join(field for field in ["hour","minute","second"] if dOpts[field]))
         return " ".join(s for s in [dateS,timeS] if len(s)>0)
-     
-    
-    
+
+
+
     def real(self):
         if self.isOneOf(["N","A"]):
-            if hasattr(self, "tab") and self.tab !=None:
-                self.realization=self.decline(False)
+            if hasattr(self, "tab") and self.tab  is not None:
+                return self.doFormat(self.decline(False))
         elif self.isA("Adv"):
-            if hasattr(self, "tab") and self.tab !=None:
-                self.realization=self.decline(False)
-            else:
+            if hasattr(self, "tab") and self.tab  is not None:
+                return self.doFormat(self.decline(False))
+            elif self.realization is None:
                 self.realization=self.lemma
         elif self.isOneOf(["C","P","Q"]):
             if self.realization is None:
                 self.realization=self.lemma
         elif self.isOneOf(["D","Pro"]):
-            if hasattr(self, "tab") and self.tab !=None:
-                self.realization=self.decline(True)
+            if hasattr(self, "tab") and self.tab is not None:
+                return self.doFormat(self.decline(True))
         elif self.isA("V"):
             return self.doFormat(self.conjugate())
         elif self.isA("DT"):
-            self.realization=self.dateFormat(self.date,self.props["dOpt"])
+            self.realization=self.dateFormat(self.date,self.getProp("dOpt"))
         elif self.isA("NO"):
             if "dOpt" in self.props:
-                opts=self.props["dOpt"]
+                self.setProp("n",self.grammaticalNumber())
+                opts=self.getProp("dOpt")
                 if "nat" in opts and opts["nat"]==True:
                     self.realization=self.numberToWord()
                 elif "ord" in opts and opts["ord"]==True:
+                    self.setProp("n","s")  # the number of an ordinal is always singular
                     self.realization=self.numberToOrdinal()
                 elif "raw" in opts and opts["raw"]==False:
                     self.realization=self.numberFormatter(opts["mprecision"])
@@ -637,28 +666,28 @@ class Terminal(Constituent):
     def toJSON(self):
         res={"terminal":self.constType,"lemma":self.lemma}
         if self.parentConst is None or self.lang!=self.parentConst.lang: # only indicate when language changes
-            res["lang"]=self.lang;
+            res["lang"]=self.lang
         if len(self.props): # do not output empty props
-            res["props"]=self.props;
-        return res;
+            res["props"]=self.props
+        return res
     
     @classmethod
-    def fromJSON(self,constType,json,lang):
+    def fromJSON(cls,constType,json,lang):
         if "lemma" in json:
             return Terminal(constType,json["lemma"],lang).setJSONprops(json)
         else:
             print("Terminal.fromJSON: no lemma found in "+str(json))
-        return self
+        return cls
         
     def toDependent(self,depName=None):
         from .Dependent import Dependent,det
         # HACK: self.parentConst in changed during transformation to refer to the parent dependennt...
-        isTopLevel = self.parentConst is None;  # we save it to use it at the end
-        if (self.isOneOf(["D", "NO"])):
+        isTopLevel = self.parentConst is None  # we save it to use it at the end
+        if self.isOneOf(["D", "NO"]):
             deprel=det(self)
         else:
-            deprel= Dependent([self], "root" if depName is None else depName);
-        if isTopLevel: deprel.cap(False);
+            deprel= Dependent([self], "root" if depName is None else depName)
+        if isTopLevel: deprel.cap(False)
         return deprel
 
 
@@ -706,7 +735,7 @@ class C(Terminal):
         super().__init__("C",lemma,lang)
 
 class DT(Terminal):
-    def __init__(self, lemma=None,lang=None):
+    def __init__(self, lemma=None,_lang=None):
         super().__init__("DT", lemma) 
             
 class NO(Terminal):
